@@ -18,16 +18,15 @@
 ##' element is a vector giving the parameter number for the random effect
 ##' coefficients in that group of random effects. These "parameter numbers"
 ##' correspond to the number of these parameters in the list of parameters in the
-##' "ss3.cor" output file.
-##' @param PAR_num_Vec Vector giving the number in the "ss3.par" vector for each
+##' ".cor" output file.
+##' @param PAR_num_Vec Vector giving the number in the ".par" vector for each
 ##' random effect coefficient.
 ##' @param Int_Group_List List where each element is a vector, providing a way of
-##' grouping different random effect groups into a single category. This is not
-##' used (but input is still required) when \code{Version=1}.
-##' @param Version Integer (options are 1, 5, and 6) giving the type of Laplace
-##' Approximation. I recommend 1.
+##' grouping different random effect groups into a single category. Although
+##' this input is still required, it is no has the former input Version has been
+##' hardwired to Version = 1.
 ##' @param StartFromPar Logical flag (TRUE or FALSE) saying whether to start each
-##' round of optimization from a "ss3.par" file (I recommend TRUE)
+##' round of optimization from a ".par" file (I recommend TRUE)
 ##' @param Intern Logical flag saying whether to display all ss3 runtime output
 ##' in the R terminal
 ##' @param ReDoBiasRamp Logical flag saying whether to re-do the bias ramp
@@ -40,8 +39,12 @@
 ##' Default is NULL, and if not explicitly specified the program will attempt to
 ##' detect these automatically based on the length of relevant lines from the CTL
 ##' file.
-##' @param systemcmd Should R call SS using "system" function intead of "shell".
+##' @param systemcmd Should R call SS using "system" function instead of "shell".
 ##' This may be required when running R in Emacs on Windows. Default = FALSE.
+##' @param exe SS executable name (excluding extension), either "ss" or "ss3".
+##' This string is used for both calling the executable and also finding the
+##' output files like ss.par. For 3.30, it should always be "ss" since the
+##' output file names are hardwired in the TPL code.
 ##' @seealso \code{\link{read.admbFit}}, \code{\link{getADMBHessian}}
 ##' @author James Thorson
 ##' @export
@@ -55,24 +58,35 @@
 #'     if("Optimization_record.txt" %in% list.files(direc)) {
 #'          file.remove(file.path(direc,"Optimization_record.txt"))
 #'     }
-#'     Opt = optimize(f=NegLogInt_Fn,
+#'     Opt <- optimize(f=NegLogInt_Fn,
 #'                    interval=c(0.001, 0.12),
 #'                    maximum=FALSE,
 #'                    File=direc,
+#'                    Input_SD_Group_Vec=1,
 #'                    CTL_linenum_List=list(127:131),
 #'                    ESTPAR_num_List=list(86:205),
 #'                    Int_Group_List=1,
 #'                    PAR_num_Vec=NA,
-#'                    Version=1, Intern=TRUE)
+#'                    Intern=TRUE)
 #'   }
 
-NegLogInt_Fn <-
-  function(File=NA, Input_SD_Group_Vec,
-           CTL_linenum_List, ESTPAR_num_List,
-           PAR_num_Vec, Int_Group_List=list(1), Version=1,
-           StartFromPar=TRUE, Intern=TRUE,
-           ReDoBiasRamp=FALSE, BiasRamp_linenum_Vec=NULL,
-           CTL_linenum_Type=NULL,systemcmd=FALSE){
+NegLogInt_Fn <- function(File=NA, Input_SD_Group_Vec,
+                         CTL_linenum_List, ESTPAR_num_List,
+                         PAR_num_Vec, Int_Group_List=list(1),
+                         StartFromPar=TRUE, Intern=TRUE,
+                         ReDoBiasRamp=FALSE, BiasRamp_linenum_Vec=NULL,
+                         CTL_linenum_Type=NULL,systemcmd=FALSE,
+                         exe="ss"){
+  # test exe input
+  if(!(exe=="ss" | exe=="ss3")){
+    # turns out 3.30 != "3.30" in R
+    warning('exe inputs other than "ss" and "ss3" may not work')
+  }
+  # frequently used files
+  parfile <- paste0(exe, ".par")
+  stdfile <- paste0(exe, ".std")
+  corfile <- paste0(exe, ".cor")
+  
   # figure out operating system
   OS <- "Mac" # don't know the version$os info for Mac
   if(length(grep("linux",version$os)) > 0) OS <- "Linux"
@@ -80,18 +94,18 @@ NegLogInt_Fn <-
 
   # Directory
   if(is.na(File)){
-      File = getwd()
+    File <- getwd()
   }
 
   if( "Iteration.txt" %in% list.files(File)){
-      Iteration <- read.table(file=file.path(File,"Iteration.txt"))[[1]]
+    Iteration <- read.table(file=file.path(File,"Iteration.txt"))[[1]]
   }else{
-      Iteration <- 0
+    Iteration <- 0
   }
 
   # Error messages
   if(ReDoBiasRamp==TRUE & is.null(BiasRamp_linenum_Vec)){
-      stop("If ReDoBiasRamp==TRUE, then BiasRamp_linenum_Vec must be specified")
+    stop("If ReDoBiasRamp==TRUE, then BiasRamp_linenum_Vec must be specified")
   }
 
   # Make sure print is high enough for when passing values to ADMB
@@ -108,21 +122,21 @@ NegLogInt_Fn <-
   #    Iteration <- max(Iterations)
 
   # Transform parameter vector
-  SD_Group_Vec = Input_SD_Group_Vec
+  SD_Group_Vec <- Input_SD_Group_Vec
 
   # Modify inputs when necessary
   if(is.null(CTL_linenum_Type)){
-    CTL_linenum_Type = rep(NA, length(SD_Group_Vec))
+    CTL_linenum_Type <- rep(NA, length(SD_Group_Vec))
   }
 
   # define some filenames with full path
   OptRecord <- file.path(File, "Optimization_record.txt")
-  ParFile   <- file.path(File, "ss3.par")
-
+  ParFile   <- file.path(File, parfile)
+  
   # Write record to file (part 1)
   if(!("Optimization_record.txt" %in% list.files(File))){
-      write("Start optimization",file=OptRecord,
-            append=FALSE)
+    write("Start optimization",file=OptRecord,
+          append=FALSE)
   }
   write("",file=OptRecord,append=TRUE)
   write(date(),file=OptRecord,append=TRUE)
@@ -131,15 +145,15 @@ NegLogInt_Fn <-
   write(paste("SD_Group_Vec",paste(SD_Group_Vec,collapse=" ")),
         file=OptRecord,append=TRUE)
 
-  # If ss3.par is availabile from the last iteration then use it as starting point
+  # If .par is availabile from the last iteration then use it as starting point
   STARTER <- SS_readstarter(file.path(File,"starter.ss"), verbose=FALSE)
-  if( paste("ss3_",Iteration-1,".par",sep="") %in% list.files(File) &
-     StartFromPar==TRUE ){
+  if( StartFromPar==TRUE &&
+     paste0(exe, "_", Iteration-1, ".par") %in% list.files(File) ){
     STARTER$init_values_src <- 1
-    PAR_0 <- scan(file.path(File,paste("ss3_",Iteration-1,".par",sep="")),
+    PAR_0 <- scan(file.path(File, paste0(exe, "_",Iteration-1,".par")),
                   comment.char="#", quiet=TRUE)
   }else{
-    STARTER$init_values_src = 0
+    STARTER$init_values_src <- 0
   }
   SS_writestarter(STARTER, dir=File, file="starter.ss", overwrite=TRUE, verbose=FALSE)
 
@@ -154,17 +168,17 @@ NegLogInt_Fn <-
       Temp <- as.vector(unlist(sapply(Temp, FUN=function(Char){strsplit(Char,"\t")[[1]]})))
       Temp <- Temp[which(Temp!="")]
       if(length(grep("#",Temp))>=1){
-        Temp = Temp[-(grep("#",Temp):length(Temp))]
+        Temp <- Temp[-(grep("#",Temp):length(Temp))]
       }
       Temp <- as.numeric(Temp)
       #print(Temp)
       #print(CTL_linenum_Type[ParI])
       if(is.na(CTL_linenum_Type[ParI])){
         if(length(Temp)==7){
-          CTL_linenum_Type[ParI] = "Short_Param"
+          CTL_linenum_Type[ParI] <- "Short_Param"
         }
         if(length(Temp)==14){
-          CTL_linenum_Type[ParI] = "Long_Penalty"
+          CTL_linenum_Type[ParI] <- "Long_Penalty"
         }
       }
       if(CTL_linenum_Type[ParI] %in% c("Short_Param","Long_Param")){
@@ -172,7 +186,7 @@ NegLogInt_Fn <-
         Temp[7] <- -1 * abs(Temp[7])
         # Modify values of PAR file for short-line values
         if( "PAR_0" %in% ls() ){
-          PAR_0[PAR_num_Vec[ParI]] = SD_Group_Vec[ParI]
+          PAR_0[PAR_num_Vec[ParI]] <- SD_Group_Vec[ParI]
         }
         #if(ParI==2){
         #  assign("Temp", value=Temp, envir=.GlobalEnv)
@@ -184,8 +198,8 @@ NegLogInt_Fn <-
       }
       CTL[CTL_linenum_List[[ParI]][CtlLineI]] <- paste(Temp, collapse=" ")
       #if(ParI==2){
-        #assign("Temp", value=Temp, envir=.GlobalEnv)
-        #stop()
+      #assign("Temp", value=Temp, envir=.GlobalEnv)
+      #stop()
       #}
     }
   }
@@ -194,14 +208,14 @@ NegLogInt_Fn <-
   # Write CTL
   writeLines(CTL, file.path(File,STARTER$ctlfile))
   if( "PAR_0" %in% ls() ){
-      write(PAR_0, file=ParFile, ncolumns=10)
+    write(PAR_0, file=ParFile, ncolumns=10)
   }
 
   # Run SS
   setwd(File)
-  command <- "ss3 -nohess -cbs 500000000 -gbs 500000000"
+  command <- paste0(exe, " -nohess -cbs 500000000 -gbs 500000000")
   if(OS!="Windows"){
-    command <- paste("./",command,sep="")
+    command <- paste0("./", command)
   }
   if(OS=="Windows" & !systemcmd){
     shell(cmd=command,intern=Intern)
@@ -212,17 +226,17 @@ NegLogInt_Fn <-
 
   # Check convergence
   Converged <- FALSE
-  if("ss3.par" %in% list.files(File)){
+  if(parfile %in% list.files(File)){
     # Move PAR files
     file.rename(from=ParFile,
-                to=file.path(File,paste("ss3_",Iteration,"-first.par",sep="")))
+                to=file.path(File,paste0(exe, "_",Iteration,"-first.par")))
     # Read and check
-    PAR <- scan(file.path(File,paste("ss3_",Iteration,"-first.par",sep="")),
+    PAR <- scan(file.path(File,paste0(exe, "_",Iteration,"-first.par")),
                 what="character", quiet=TRUE)
     if( ifelse(is.na(as.numeric(PAR[11])),FALSE,as.numeric(PAR[16])<1) ){
       Converged <- TRUE
     }else{
-      write(paste("*** Optimization ",1," didn't converge ***",sep=""),
+      write(paste0("*** Optimization ",1," didn't converge ***"),
             file=OptRecord,append=TRUE)
     }
   }
@@ -238,15 +252,15 @@ NegLogInt_Fn <-
     while(Converged==FALSE & PreviousIteration<=Iteration){
       # Read in original estimate
       if(PreviousIteration==0){
-        PAR_0 <- scan(file.path(File, paste("ss3_",PreviousIteration,".par", sep="")),
+        PAR_0 <- scan(file.path(File, paste0(exe, "_",PreviousIteration,".par")),
                       comment.char="#", quiet=TRUE)
       }
       if(PreviousIteration>=1 & PreviousIteration<Iteration){
-        PAR_0 <- scan(file.path(File, paste("ss3_",PreviousIteration,"-first.par",sep="")),
+        PAR_0 <- scan(file.path(File, paste0(exe, "_",PreviousIteration,"-first.par")),
                       comment.char="#", quiet=TRUE)
       }
-      if(PreviousIteration==Iteration & "ss3_init.par"%in%list.files(File)){
-        PAR_0 <- scan(file.path(File, "ss3_init.par"), comment.char="#", quiet=TRUE)
+      if(PreviousIteration==Iteration & paste0(exe, "_init.par") %in% list.files(File)){
+        PAR_0 <- scan(file.path(File, paste0(exe, "_init.par")), comment.char="#", quiet=TRUE)
       }
       # Modify values of PAR file for short-line values
       for(ParI in 1:length(SD_Group_Vec)){
@@ -260,9 +274,9 @@ NegLogInt_Fn <-
         write(PAR_0, file=ParFile, ncolumns=10)
       }
       # Run SS
-      command <- "ss3 -nohess -cbs 500000000 -gbs 500000000"
+      command <- paste0(exe, " -nohess -cbs 500000000 -gbs 500000000")
       if(OS!="Windows"){
-        command <- paste("./",command,sep="")
+        command <- paste0("./", command)
       }
       if(OS=="Windows" & !systemcmd){
         shell(cmd=command,intern=Intern)
@@ -271,23 +285,24 @@ NegLogInt_Fn <-
       }
       Sys.sleep(1)
       # Check convergence
-      if("ss3.par" %in% list.files(File)){
+      if(parfile %in% list.files(File)){
         # Move PAR files
         file.copy(from=ParFile,
-                  to=file.path(File,paste("ss3_",Iteration,"-first.par",sep="")),
+                  to=file.path(File,paste0(exe, "_",Iteration,"-first.par")),
                   overwrite=TRUE)
         file.remove(ParFile)
         # Read and check
-        PAR <- scan(file.path(File, paste("ss3_",Iteration,"-first.par",sep="")),
+        PAR <- scan(file.path(File, paste0(exe, "_",Iteration,"-first.par")),
                     what="character", quiet=TRUE)
         if( ifelse(is.na(as.numeric(PAR[11])),FALSE,as.numeric(PAR[16])<1) ){
           Converged <- TRUE
-          write(paste("*** Optimization ",2,"-",PreviousIteration," did converge ***",
-                      sep=""),file=OptRecord,
-                append=TRUE)
+          write(paste0("*** Optimization ",2,"-", PreviousIteration,
+                       " did converge ***"),
+                file=OptRecord, append=TRUE)
         }else{
-          write(paste("*** Optimization ",2,"-",PreviousIteration," didn't converge ***",sep=""),
-                file=OptRecord,append=TRUE)
+          write(paste0("*** Optimization ",2,"-",PreviousIteration,
+                       " didn't converge ***"),
+                file=OptRecord, append=TRUE)
         }
       }
       # Increment
@@ -302,29 +317,31 @@ NegLogInt_Fn <-
     STARTER$init_values_src <- 1
     SS_writestarter(STARTER, dir=File, file="starter.ss", overwrite=TRUE,
                     verbose=FALSE)
-    file.copy(from=file.path(File, paste("ss3_",Iteration,"-first.par",sep="")),
+    file.copy(from=file.path(File, paste0(exe, "_",Iteration,"-first.par")),
               to=ParFile, overwrite=TRUE)
-    if(file.exists(file.path(File, "ss3.std"))) { file.remove(file.path(File, "ss3.std")) }
-    command <- "ss3 -maxfn 0 -cbs 500000000 -gbs 500000000"
+    if(file.exists(file.path(File, stdfile))) {
+      file.remove(file.path(File, stdfile))
+    }
+    command <- paste0(exe, " -maxfn 0 -cbs 500000000 -gbs 500000000")
     if(OS!="Windows"){
-        command <- paste("./",command,sep="")
+      command <- paste0("./", command)
     }
     if(OS=="Windows" & !systemcmd){
-        shell(cmd=command,intern=Intern)
+      shell(cmd=command,intern=Intern)
     }else{
-        system(command,intern=Intern)
+      system(command,intern=Intern)
     }
     Sys.sleep(1)
 
     # Estimate new bias ramp
     if( ReDoBiasRamp==TRUE
-       & "ss3.std" %in% list.files(File)
-       & file.info(file.path(File, "ss3.std"))$size>0 ){
+       & stdfile %in% list.files(File)
+       & file.info(file.path(File, stdfile))$size>0 ){
       # try reading output
       SsOutput <- try(SS_output(File, covar=TRUE, forecast=FALSE, verbose=F, printstats=F), silent=TRUE)
       if( class(SsOutput)!='try-error' ){
         BiasRamp <- SS_fitbiasramp(SsOutput, altmethod="psoptim", print=FALSE, plot=FALSE)
-        file.remove(file.path(File, "ss3.std"))
+        file.remove(file.path(File, stdfile))
         # Put into CTL
         CTL <- readLines(file.path(File, STARTER$ctlfile))
         CTL[BiasRamp_linenum_Vec] <- apply(BiasRamp$df, MARGIN=1, FUN=paste, collapse=" ")
@@ -332,7 +349,7 @@ NegLogInt_Fn <-
         # Re-run to get Hessian
         command <- "ss3 -cbs 500000000 -gbs 500000000"
         if(OS!="Windows"){
-          command <- paste("./",command,sep="")
+          command <- paste0("./", command)
         }
         if(OS=="Windows" & !systemcmd){
           shell(cmd=command,intern=Intern)
@@ -346,37 +363,37 @@ NegLogInt_Fn <-
 
   # Check for STD
   Converged <- FALSE
-  if( "ss3.std"%in%list.files(File) & file.info(file.path(File, "ss3.std"))$size>0 ){
-    Converged=TRUE
+  if( stdfile%in%list.files(File) & file.info(file.path(File, stdfile))$size>0 ){
+    Converged <- TRUE
   }
 
   # If STD exists, then approximate marginal likelihood
   if(Converged==TRUE){
     # Save objects for replicating analysis
     file.rename(from=ParFile,
-                to=file.path(File, paste("ss3_",Iteration,".par",sep="")))
-    file.rename(from=file.path(File,"ss3.std"),
-                to=file.path(File, paste("ss3_",Iteration,".std",sep="")))
-    file.rename(from=file.path(File,"ss3.cor"),
-                to=file.path(File, paste("ss3_",Iteration,".cor",sep="")))
+                to=file.path(File, paste0(exe, "_",Iteration,".par")))
+    file.rename(from=file.path(File, stdfile),
+                to=file.path(File, paste0(exe, "_",Iteration,".std")))
+    file.rename(from=file.path(File, corfile),
+                to=file.path(File, paste0(exe, "_",Iteration,".cor")))
     file.rename(from=file.path(File,"admodel.hes"),
-                to=file.path(File, paste("admodel_",Iteration,".hes",sep="")))
+                to=file.path(File, paste0("admodel_",Iteration,".hes")))
     file.rename(from=file.path(File,"Report.sso"),
-                to=file.path(File, paste("Report_",Iteration,".sso",sep="")))
+                to=file.path(File, paste0("Report_",Iteration,".sso")))
     file.copy(from=file.path(File,STARTER$datfile),
-              to=file.path(File, paste(STARTER$datfile,"_",Iteration,".dat",sep="")))
+              to=file.path(File, paste0(STARTER$datfile,"_",Iteration,".dat")))
     file.copy(from=file.path(File,STARTER$ctlfile),
-              to=file.path(File, paste(STARTER$ctlfile,"_",Iteration,".ctl",sep="")))
+              to=file.path(File, paste0(STARTER$ctlfile,"_",Iteration,".ctl")))
 
     # Read in some stuff
-    STD <- scan(file.path(File, paste("ss3_",Iteration,".std",sep="")),
+    STD <- scan(file.path(File, paste0(exe, "_",Iteration,".std")),
                 what="character", quiet=TRUE)
     STD <- data.frame(matrix(STD[-c(1:(which(STD=="1")[1]-1))], ncol=4, byrow=TRUE),
-        stringsAsFactors=FALSE)
-    PAR <- scan(file.path(File, paste("ss3_",Iteration,".par",sep="")),
-                          comment.char="#", quiet=TRUE)
-    DIAG <- read.admbFit(file.path(File, paste("ss3_",Iteration,sep="")))
-    HESS <- getADMBHessian(File=File, FileName=paste("admodel_",Iteration,".hes",sep=""))
+                      stringsAsFactors=FALSE)
+    PAR <- scan(file.path(File, paste0(exe, "_",Iteration,".par")),
+                comment.char="#", quiet=TRUE)
+    DIAG <- read.admbFit(file.path(File, paste0(exe, "_",Iteration)))
+    HESS <- getADMBHessian(File=File, FileName=paste0("admodel_",Iteration,".hes"))
     # Calculate Hessian
     cov <- corpcor::pseudoinverse(HESS$hes)
     scale <- HESS$scale
@@ -408,15 +425,16 @@ NegLogInt_Fn <-
     }
     # Add in constant of proportionality for recruitment (i.e. to account for
     # Rick's bias-correction ramp)
-    BiasAdj <- readLines(file.path(File, paste("Report_",Iteration,".sso",sep="")))
-    if(BiasAdj[1] %in% c("#V3.24U","#V3.24V")){
+    BiasAdj <- readLines(file.path(File, paste0("Report_",Iteration,".sso")))
+    # starting with 3.24U, a new output was added 3 lines after SPAWN_RECRUIT
+    if(grep("Bmsy/Bzero", BiasAdj[pmatch("SPAWN_RECRUIT", BiasAdj) + 3])==1){
       shift <- 8
     }else{
       shift <- 7
     }
     BiasAdjStart <- pmatch("SPAWN_RECRUIT",BiasAdj) + shift
-    BiasAdjTable <- read.table(file.path(File, paste("Report_",Iteration,".sso",sep="")),
-        header=TRUE, nrows=2, skip=BiasAdjStart, comment.char="#")
+    BiasAdjTable <- read.table(file.path(File, paste0("Report_",Iteration,".sso")),
+                               header=TRUE, nrows=2, skip=BiasAdjStart, comment.char="#")
     SigmaR <- as.numeric(strsplit(BiasAdj[BiasAdjStart-4]," ")[[1]][1])
     # Deal with eras
     RecDevPen <- matrix(NA,nrow=3,ncol=2,dimnames=list(c("Early","Main","Forecast"),
@@ -432,9 +450,9 @@ NegLogInt_Fn <-
     RecDevPen['Forecast','negative-Rick'] <- 0
     # Add into NLL and record
     NLL <- NLL + sum(RecDevPen)
-    write.table(RecDevPen, file=file.path(File, paste("ss3_",Iteration,".pen",sep="")))
+    write.table(RecDevPen, file=file.path(File, paste0(exe, "_",Iteration,".pen")))
     write(c("","sum(RecDevPen) = ",sum(RecDevPen)),
-          file=file.path(File, paste("ss3_",Iteration,".pen",sep="")), append=TRUE)
+          file=file.path(File, paste0(exe, "_",Iteration,".pen")), append=TRUE)
 
     # Approximate integral using Laplace Approximation
     Int_num_List <- vector("list", length=length(Int_Group_List))
@@ -444,26 +462,27 @@ NegLogInt_Fn <-
       if( length(unlist(ESTPAR_num_List[Int_Group_List[[IntI]]])) > 0 ){
         # Determine indices for integral
         Int_num_List[[IntI]] <- unlist(ESTPAR_num_List[Int_Group_List[[IntI]]])
+
         #Version 1 -- use full hessian
-        if(Version==1){
-          if(IntI==1){
-            LnDet[IntI] <- determinant(Hess, logarithm=TRUE)$modulus[[1]]
-          }
-          if(IntI>=2) LnDet[IntI] <- 0
-        }
-        #Version 5 -- use back-transformed hessian, use subset
-        if(Version==5){
-          #Hess2 <- cov * solve(scale %o% scale)
-          #Which2 <- 38 + 1:46
-          LnDet[IntI] <- determinant(Hess[Int_num_List[[IntI]],Int_num_List[[IntI]]],
-                   logarithm=TRUE)$modulus[[1]]
-        }
-        #Version 6 -- use subset of covariance calculated from COR file
-        if(Version==6){
-          Cov <- DIAG$cov
-          LnDet[IntI] <- -1 * determinant(Cov[Int_num_List[[IntI]],Int_num_List[[IntI]]],
-                                         logarithm=TRUE)$modulus[[1]]
+        if(IntI==1){
+          LnDet[IntI] <- determinant(Hess, logarithm=TRUE)$modulus[[1]]
         }
+        if(IntI>=2) LnDet[IntI] <- 0
+
+        #### alternative versions taken out based on recommendation from Jim Thorson
+        ## #Version 5 -- use back-transformed hessian, use subset
+        ## if(Version==5){
+        ##   #Hess2 <- cov * solve(scale %o% scale)
+        ##   #Which2 <- 38 + 1:46
+        ##   LnDet[IntI] <- determinant(Hess[Int_num_List[[IntI]],Int_num_List[[IntI]]],
+        ##            logarithm=TRUE)$modulus[[1]]
+        ## }
+        ## #Version 6 -- use subset of covariance calculated from COR file
+        ## if(Version==6){
+        ##   Cov <- DIAG$cov
+        ##   LnDet[IntI] <- -1 * determinant(Cov[Int_num_List[[IntI]],Int_num_List[[IntI]]],
+        ##                                  logarithm=TRUE)$modulus[[1]]
+        ## }
       }
     }
 
