@@ -25,14 +25,33 @@
 #'   \code{linenums} or \code{strings}.
 #' @param repeat.vals If multiple parameter lines match criteria, repeat the
 #'   \code{newvals} input for each line.
-#' @param estimate Vector of TRUE/FALSE for which changed parameters are to be
-#'   estimated. Default=FALSE. Can also be \code{NULL}.
-#' @param newlos Vector of new lo bounds. Default=NULL.
+#' @param estimate Optional vector or single value of TRUE/FALSE for which
+#'   parameters are to be estimated. Changes sign of phase to be positive or
+#'   negative. Default \code{NULL} causes no change to phase.
+#' @param newlos Vector of new lower bounds. Default=NULL.
 #'   The vector can contain \code{NA} values, which will assign the original
 #'   value to the given parameter but change the remainder parameters, where
 #'   the vector of values needs to be in the same order as either
 #'   \code{linenums} or \code{strings}.
-#' @param newhis Vector of new hi bounds. Must be the same length as newhis
+#' @param newhis Vector of new high bounds. Must be the same length as newhis
+#'   Default=NULL.
+#'   The vector can contain \code{NA} values, which will assign the original
+#'   value to the given parameter but change the remainder parameters, where
+#'   the vector of values needs to be in the same order as either
+#'   \code{linenums} or \code{strings}.
+#' @param newprior Vector of new prior values. 
+#'   Default=NULL.
+#'   The vector can contain \code{NA} values, which will assign the original
+#'   value to the given parameter but change the remainder parameters, where
+#'   the vector of values needs to be in the same order as either
+#'   \code{linenums} or \code{strings}.
+#' @param newprsd Vector of new prior sd values. 
+#'   Default=NULL.
+#'   The vector can contain \code{NA} values, which will assign the original
+#'   value to the given parameter but change the remainder parameters, where
+#'   the vector of values needs to be in the same order as either
+#'   \code{linenums} or \code{strings}.
+#' @param newprtype Vector of new prior type. 
 #'   Default=NULL.
 #'   The vector can contain \code{NA} values, which will assign the original
 #'   value to the given parameter but change the remainder parameters, where
@@ -45,32 +64,43 @@
 #'   on or off and not change the phase in which they are estimated use
 #'   \code{estimate = TRUE} or \code{estimate = FALSE}, respectively.
 #'   The vector can contain \code{NA} values, which will assign the original
-#'   value to the given parameter but change the remainder parameters, where
+#'   value to the given parameter but change the remaining parameters, where
 #'   the vector of values needs to be in the same order as either
 #'   \code{linenums} or \code{strings}.
 #' @param verbose More detailed output to command line. Default=TRUE.
-#' @author Ian Taylor, Christine Stawitz
+#' @author Ian Taylor, Christine Stawitz, Chantel Wetzel
 #' @seealso \code{\link{SS_parlines}}, \code{\link{SS_profile}}
 #' @export
-#' @keywords data manip
 #' @examples
 #'
 #' \dontrun{
-#' SS_changepars(dir='Y:/ss/SSv3.03a/Simple/',ctlfile='Control.SS_New',
-#'               strings=c('SR_steep','SR_sigmaR'),newvals=c(.35,.6))
-#' # [1] wrote new file to Control_Modified.SS
-#' #    oldvals newvals oldphase newphase     comment
-#' # 1 0.609048    0.35        4       -4  # SR_steep
-#' # 2 0.600000    0.60       -4       -4 # SR_sigmaR
+#' SS_changepars(dir='C:/ss/SSv3.30.03.05_May11/Simple - Copy',
+#'               strings=c("steep","sigmaR"), newvals=c(.4,.6))
+#' ## parameter names in control file matching input vector 'strings' (n=2):
+#' ## [1] "SR_BH_steep" "SR_sigmaR"  
+#' ## These are the ctl file lines as they currently exist:
+#' ##     LO HI     INIT PRIOR PR_type SD PHASE env-var use_dev dev_minyr dev_maxyr
+#' ## 95 0.2  1 0.613717   0.7    0.05  1     4       0       0         0         0
+#' ## 96 0.0  2 0.600000   0.8    0.80  0    -4       0       0         0         0
+#' ##    dev_stddev Block Block_Fxn       Label Linenum
+#' ## 95          0     0         0 SR_BH_steep      95
+#' ## 96          0     0         0   SR_sigmaR      96
+#' ## line numbers in control file (n=2):
+#' ## [1] 95 96
+#' ## 
+#' ## wrote new file to control_modified.ss with the following changes:
+#' ##    oldvals newvals oldphase newphase oldlos newlos oldhis newhis       comment
+#' ## 1 0.613717     0.4        4       -4    0.2    0.2      1      1 # SR_BH_steep
+#' ## 2 0.600000     0.6       -4       -4    0.0    0.0      2      2   # SR_sigmaR
 #' }
-#'
 SS_changepars <-
 function(
          dir=NULL,
          ctlfile="control.ss_new",
          newctlfile="control_modified.ss",
          linenums=NULL, strings=NULL, newvals=NULL, repeat.vals=FALSE,
-         newlos=NULL, newhis=NULL, estimate=FALSE, verbose=TRUE,
+         newlos=NULL, newhis=NULL, newprior=NULL, newprsd=NULL, newprtype=NULL,
+         estimate=NULL, verbose=TRUE,
          newphs = NULL
          )
 {
@@ -82,24 +112,37 @@ function(
   fullctlfile <- file.path(dir, ctlfile)
   ctl <- readLines(fullctlfile)
 
-  # check for valid input
+# check for valid input
+  inargs <- list("newvals" = newvals, "newlos" = newlos, "newhis" = newhis, 
+    "newprior" = newprior, "newprsd" = newprsd, "newprtype" = newprtype, 
+    "estimate" = estimate, "newphs" = newphs)
   if(is.null(linenums) & !is.null(strings) & class(strings)=="character")
   {
     # get table of parameter lines
     ctltable <- SS_parlines(ctlfile=fullctlfile)
+    
     # list of all parameter labels
     allnames <- ctltable$Label
     # empty list of "good" labels to be added to
-    goodnames <- NULL
+    goodnames <- list()
     # if strings are provided, look for matching subset of labels
     if(!is.null(strings)){
       # loop over vector of strings to add to goodnames vector
       for(i in 1:length(strings)){
         # fixed matching on string
-        goodnames <- c(goodnames, allnames[grep(strings[i], allnames, fixed=TRUE)])
+        goodnames[[i]] <- allnames[grep(strings[i], allnames, fixed=TRUE)]
       }
       # remove duplicates and print some feedback
-      goodnames <- unique(goodnames)
+      if (any(duplicated(unlist(goodnames))) & 
+        (repeat.vals & any(sapply(inargs, length) > 1))) {
+        stop("Entries in 'strings' did not map to unique parameters and\n",
+          "it is unclear how to order the par names to match the order\n",
+          "of other arguments provided to SS_changepars.\n",
+          "E.g., strings = c('CV', 'Mal') each return 'CV_young_Mal_GP_1'\n",
+          "and should be changed to strings = c('young_Fem', 'old_Fem', 'Mal')\n",
+          "to get all CV and all Male parameters.")
+      }
+      goodnames <- unique(unlist(goodnames))
       if(verbose){
         cat("parameter names in control file matching input vector 'strings' (n=",
             length(goodnames),"):\n",sep="")
@@ -125,69 +168,34 @@ function(
   ctlsubset <- ctl[linenums]
   if(verbose){
     cat("line numbers in control file (n=",length(linenums),"):\n",sep="")
-    print(linenums)
+    cat(paste(linenums, collapse = ", "))
   }
   # define objects to store changes
   newctlsubset <- NULL
   cmntvec <- NULL
   nvals <- length(linenums)
-  oldvals <- oldlos <- oldhis <- oldphase <- newphase <- rep(NA, nvals)
-
+  # make vectors of NA values for old and new quantities
+  oldvals <- oldlos <- oldhis <- oldphase <- rep(NA, nvals)
+  oldprior <- oldprsd <- oldprtype <- newphase <- rep(NA, nvals)
   # check all inputs
   # check values and make repeat if requested
-  if (!is.null(newvals) & length(newvals)!=nvals){
-    if (repeat.vals){
-      newvals <- rep(newvals, nvals)
-    }else{
-      stop("'newvals' and either 'linenums' or 'strings' should have",
-           "the same number of elements")
+  for (ii in names(inargs)) {
+    tmp <- get(ii)
+    if (is.null(tmp)) next
+    if (is.data.frame(tmp) & ii!="estimate") tmp <- as.numeric(tmp)
+    if (length(tmp)!=nvals & repeat.vals) {
+      if (length(tmp) > 1) stop("SS_changepars doesn't yet accommodate ",
+        "repeat.vals=TRUE and of length(.) > 1")
+      assign(ii, rep(tmp, nvals))
+    }
+    if (length(get(ii))!=nvals) {
+      stop(paste0("'", ii, "'"), " and either 'linenums' or 'strings'",
+        " should have the same number of elements,\n",
+        "instead of ", length(get(ii)), " and ", length(linenums), ".\n",
+        "Note: a string can map to multiple parameters, here are your pars,\n",
+        paste(goodnames, collapse = "\n"))
     }
   }
-  # check bounds
-  # lower and upper bounds don't yet have option for repeat.vals=TRUE
-  if (!is.null(newlos) & length(newlos) != nvals) {
-    stop("'newlos' and either 'linenums' or 'strings' should have",
-         "the same number of elements")
-  }
-  if (!is.null(newhis) & length(newhis) != nvals) {
-    stop("'newhis' and either 'linenums' or 'strings' should have",
-         "the same number of elements")
-  }
-  if (is.data.frame(newlos)){
-    newlos <- as.numeric(newlos)
-  }
-  if (is.data.frame(newhis)){
-    newhis <- as.numeric(newhis)
-  }
-  if (!is.null(estimate)){
-    if (!(length(estimate) %in% c(1,nvals))){
-      stop("'estimate' should have 1 element or same number as 'newvals'")
-    }
-    if (length(estimate)==1){
-      estimate <- rep(estimate, nvals)
-    }
-  }
-  if (!is.null(newphs)){
-    if (!(length(newphs) %in% c(1, nvals))){
-      stop("'newphs' should have 1 element or same number as 'newvals'")
-    }
-    if (length(newphs)==1){
-      newphs <- rep(newphs, nvals)
-    }
-  }
-  if (is.data.frame(newvals)){
-    newvals <- as.numeric(newvals)
-  }
-  #### if inputs are NULL, allow newlows and newhis to be replaced by old values
-  ## if (is.null(newlos)){
-  ##   stop("Nothing input for 'newlos'")
-  ## }
-  ## if (is.null(newhis)){
-  ##   stop("Nothing input for 'newhis'")
-  ## }
-  ## if(is.null(newvals)){
-  ##   stop("Nothing input for 'newvals'")
-  ## }
 
   navar <- c(NA, "NA", "NAN", "Nan")
 
@@ -203,7 +211,7 @@ function(
     vecstrings <- strsplit(splitline[1],split="[[:blank:]]+")[[1]]
     vec <- as.numeric(vecstrings[vecstrings!=""])
     if(max(is.na(vec))==1){
-      stop("There's a problem with a non-numeric value in line",linenums[i])
+      stop("There's a problem with a non-numeric value in line ",linenums[i])
     }
     # store information on old value and replace with new value (unless NULL)
     oldvals[i] <- vec[3]
@@ -228,6 +236,27 @@ function(
       }
       vec[2] <- newhis[i]
     }
+    oldprior[i] <- vec[4]
+    oldprsd[i]  <- vec[5]
+    oldprtype[i]<- vec[6]
+    if (!is.null(newprior)){
+      if (newprior[i] %in% navar) {
+        newprior[i] <- vec[4]
+      }
+      vec[4] <- newprior[i]
+    }
+    if (!is.null(newprsd)){
+      if (newprsd[i] %in% navar) {
+        newprsd[i] <- vec[5]
+      }
+      vec[5] <- newprsd[i]
+    }
+    if (!is.null(newprtype)){
+      if (newprtype[i] %in% navar) {
+        newprtype[i] <- vec[6]
+      }
+      vec[6] <- newprtype[i]
+    }
 
     # change phase (unless NULL)
     oldphase[i] <- as.numeric(vec[7])
@@ -263,6 +292,7 @@ function(
   if(verbose){
     cat('\nwrote new file to',newctlfile,'with the following changes:\n')
   }
+
   # if no changed made, repeat old values in output
   if (is.null(newvals)){
     newvals <- oldvals
@@ -273,8 +303,19 @@ function(
   if (is.null(newhis)){
     newhis <- oldhis
   }
+  if (is.null(newprior)){
+    newprior <- oldprior
+  }
+  if (is.null(newprsd)){
+    newprsd <- oldprsd
+  }
+  if (is.null(newprtype)){
+    newprtype <- oldprtype
+  }
   results <- data.frame(oldvals, newvals, oldphase, newphase,
-                        oldlos, newlos, oldhis, newhis, comment=cmntvec)
+                        oldlos, newlos, oldhis, newhis, 
+                        oldprior, newprior, oldprsd, newprsd, 
+                        oldprtype, newprtype, comment=cmntvec)
   # output table of changes
   if (is.null(newvals)) {
     newvals <- NA
