@@ -14,10 +14,8 @@
 #' @author Ian Taylor
 #' @export
 #' @seealso \code{\link{SS_readstarter}}, \code{\link{SS_readforecast}},
-#' \code{\link{SS_readctl}}, \code{\link{SS_writestarter}},
-#' \code{\link{SS_writeforecast}}, \code{\link{SS_writedat}},
-#' \code{\link{SS_writectl}}
-#' @keywords data manip
+#' \code{\link{SS_writestarter}},
+#' \code{\link{SS_writeforecast}}, \code{\link{SS_writedat}}
 SS_writestarter <- function(mylist, dir=NULL, file="starter.ss",
                             overwrite=FALSE, verbose=TRUE, warn=TRUE){
   if(verbose) cat("running SS_writestarter\n")
@@ -98,10 +96,15 @@ SS_writestarter <- function(mylist, dir=NULL, file="starter.ss",
   wl("depl_denom_frac")
   wl("SPR_basis")
   wl("F_report_units")
-  if(mylist$F_report_units==4){
+  if(mylist$F_report_units %in% 4:5){
     cat(mylist[["F_age_range"]],"#_F_age_range\n")
   }
   wl("F_report_basis")
+  # only write ALK_tolerance if this is SSv3.30 (value didn't exist in 3.24)
+  if(mylist$final==3.3){
+    wl("MCMC_output_detail")
+    wl("ALK_tolerance")
+  }
   writeLines("#")
   wl("final")
 
