@@ -1,9 +1,9 @@
 #' Plot mean weight data and fits.
-#' 
+#'
 #' Plot mean weight data and fits from Stock Synthesis output. Intervals are
 #' based on T-distributions as specified in model.
-#' 
-#' 
+#'
+#'
 #' @param replist list created by \code{SS_output}
 #' @param ymax Optional input to override default ymax value.
 #' @param subplots Vector of which plots to make (1 = data only, 2 = with fit).
@@ -30,7 +30,6 @@
 #' @author Ian Taylor, Ian Stewart
 #' @export
 #' @seealso \code{\link{SS_plots}}, \code{\link{SS_output}}
-#' @keywords hplot
 SSplotMnwt <-
   function(replist, subplots=1:2, ymax=NULL,
            plot=TRUE, print=FALSE,
@@ -49,10 +48,11 @@ SSplotMnwt <-
            cex.main=1,
            plotdir="default", verbose=TRUE)
 {
-  pngfun <- function(file,caption=NA){
-    png(filename=file,width=pwidth,height=pheight,
-        units=punits,res=res,pointsize=ptsize)
-    plotinfo <- rbind(plotinfo,data.frame(file=file,caption=caption))
+  # subfunction to write png files
+  pngfun <- function(file, caption=NA){
+    png(filename=file.path(plotdir, file),
+        width=pwidth, height=pheight, units=punits, res=res, pointsize=ptsize)
+    plotinfo <- rbind(plotinfo, data.frame(file=file, caption=caption))
     return(plotinfo)
   }
   plotinfo <- NULL
@@ -61,20 +61,30 @@ SSplotMnwt <-
   mnwgt         <- replist$mnwgt
   FleetNames    <- replist$FleetNames
   DF_mnwgt      <- replist$DF_mnwgt
+  nfleets       <- replist$nfleets
+  SS_versionshort <- replist$SS_versionshort
 
+
+  if(fleets[1]=="all") fleets <- 1:nfleets
   if(fleetnames[1]=="default") fleetnames <- FleetNames
   if(plotdir=="default") plotdir <- replist$inputs$dir
 
   # mean body weight observations ###
   if(!is.na(mnwgt)[1]){
-    for(ifleet in intersect(fleets,unique(mnwgt$FleetNum))){
-      usemnwgt <- mnwgt[mnwgt$FleetNum==ifleet & mnwgt$Obs>0,]
-      usemnwgt$Mkt <- usemnwgt$Mkt
-      for(j in unique(mnwgt$Mkt)){
-        yr <- usemnwgt$Yr[usemnwgt$Mkt==j]
-        ob <- usemnwgt$Obs[usemnwgt$Mkt==j]
-        cv <- usemnwgt$CV[usemnwgt$Mkt==j]
-        ex <- usemnwgt$Exp[usemnwgt$Mkt==j]
+    for(ifleet in intersect(fleets,unique(mnwgt$Fleet))){
+      # usemnwgt is subset of mnwgt for the particular fleet
+      usemnwgt <- mnwgt[mnwgt$Fleet==ifleet & mnwgt$Obs>0,]
+      if(SS_versionshort == "3.30"){
+        usemnwgt$Part <- usemnwgt$Part
+      } else {
+        usemnwgt$Part <- usemnwgt$Mkt
+      }
+      FleetName <- fleetnames[ifleet]
+      for(j in unique(usemnwgt$Part)){
+        yr <- usemnwgt$Yr[usemnwgt$Part==j]
+        ob <- usemnwgt$Obs[usemnwgt$Part==j]
+        cv <- usemnwgt$CV[usemnwgt$Part==j]
+        ex <- usemnwgt$Exp[usemnwgt$Part==j]
         xmin <- min(yr)-3
         xmax <- max(yr)+3
         liw <- -ob*cv*qt(0.025,DF_mnwgt) # quantile of t-distribution
@@ -87,7 +97,7 @@ SSplotMnwt <-
         titlepart <- labels[2]
         if(j==2) titlepart <- labels[3]
         if(j==0) titlepart <- labels[4]
-        ptitle <- paste(labels[6],titlepart,labels[7],fleetnames[ifleet],sep=" ")
+        ptitle <- paste(labels[6],titlepart,labels[7],FleetName,sep=" ")
         ylab <- labels[5]
 
         # wrap up plot command in function
@@ -100,12 +110,24 @@ SSplotMnwt <-
         }
 
         # make plots
-        if(!datplot) subplots <- setdiff(subplots,1) # don't do subplot 1 if datplot=FALSE
+        if(!datplot){
+          subplots <- setdiff(subplots,1) # don't do subplot 1 if datplot=FALSE
+        }
         for(isubplot in subplots){ # loop over subplots (data only or with fit)
-          if(isubplot==1) addfit <- FALSE else addfit <- TRUE
-          if(plot) bdywtfunc(addfit=addfit)
+          if(isubplot==1){
+            addfit <- FALSE
+          }else{
+            addfit <- TRUE
+          }
+          if(plot){
+            bdywtfunc(addfit=addfit)
+          }
           if(print){
-            file <- paste(plotdir,"bodywtfit_flt",fleetnames[ifleet],".png",sep="")
+            if(!addfit){
+              file <- paste0("bodywt_data_flt",FleetName,".png")
+            }else{
+              file <- paste0("bodywt_fit_flt",FleetName,".png")
+            }
             caption <- ptitle
             plotinfo <- pngfun(file=file, caption=caption)
             bdywtfunc(addfit=addfit)
