@@ -30,7 +30,6 @@
 #' @author Ian Stewart, Ian Taylor
 #' @export
 #' @seealso \code{\link{SS_plots}}, \code{\link{SS_output}}
-#' @keywords hplot
 SSplotSPR <-
   function(replist,add=FALSE,plot=TRUE,print=FALSE,
            uncertainty=TRUE,
@@ -46,10 +45,11 @@ SSplotSPR <-
 {
   # plot SPR-related quantities
   
-  pngfun <- function(file,caption=NA){
-    png(filename=file,width=pwidth,height=pheight,
-        units=punits,res=res,pointsize=ptsize)
-    plotinfo <- rbind(plotinfo,data.frame(file=file,caption=caption))
+  # subfunction to write png files
+  pngfun <- function(file, caption=NA){
+    png(filename=file.path(plotdir, file),
+        width=pwidth, height=pheight, units=punits, res=res, pointsize=ptsize)
+    plotinfo <- rbind(plotinfo, data.frame(file=file, caption=caption))
     return(plotinfo)
   }
   plotinfo <- NULL
@@ -69,13 +69,13 @@ SSplotSPR <-
   if(btarg=="default") btarg <- replist$btarg
 
   # choose which points to plot
-  good <- sprseries$Year <= endyr
+  good <- sprseries$Yr <= endyr
   if(forecastplot) good <- rep(TRUE,nrow(sprseries))
   
   sprfunc <- function(){
-    if(!add) plot(0,xlab=labels[1],ylab=labels[2],xlim=range(sprseries$Year[good]),
+    if(!add) plot(0,xlab=labels[1],ylab=labels[2],xlim=range(sprseries$Yr[good]),
                   ylim=c(0,max(1,max(sprseries$spr[!is.na(sprseries$spr)]))),type="n")
-    lines(sprseries$Year[good],sprseries$spr[good],type="o",col=col2)
+    lines(sprseries$Yr[good],sprseries$spr[good],type="o",col=col2)
     if(sprtarg>0) abline(h=sprtarg,col=col4,lty=2)
     abline(h=0,col="grey")
     abline(h=1,col="grey")
@@ -84,7 +84,7 @@ SSplotSPR <-
   if(1 %in% subplots){
     if(plot) sprfunc()
     if(print){
-      file <- file.path(plotdir,"/SPR1_series.png")
+      file <- "SPR1_series.png"
       caption <- "Timeseries of SPR"
       plotinfo <- pngfun(file=file, caption=caption)
       sprfunc()
@@ -96,9 +96,9 @@ SSplotSPR <-
   if(nseasons>1) cat("Skipped additional SPR plots because they're not yet configured for multi-season models\n")
   if(nseasons==1){ 
     sprfunc2 <- function(){
-      if(!add) plot(0,xlim=range(sprseries$Year[good]),
+      if(!add) plot(0,xlim=range(sprseries$Yr[good]),
                     xlab=labels[1],ylab=labels[3],ylim=c(0,1),type="n")
-      lines(sprseries$Year[good],(1-sprseries$spr[good]),type="o",col=col2)
+      lines(sprseries$Yr[good],(1-sprseries$spr[good]),type="o",col=col2)
       if(sprtarg>0) abline(h=(1-sprtarg),col=col4,lty=2)
       abline(h=0,col="grey")
       abline(h=1,col="grey")}
@@ -106,7 +106,7 @@ SSplotSPR <-
     if(2 %in% subplots){
       if(plot) sprfunc2()
       if(print){
-        file <- file.path(plotdir,"/SPR2_minusSPRseries.png")
+        file <- "SPR2_minusSPRseries.png"
         caption <- "Timeseries of 1-SPR"
         plotinfo <- pngfun(file=file, caption=caption)
         sprfunc2()
@@ -117,8 +117,8 @@ SSplotSPR <-
     if(!uncertainty | sprtarg<=0){
       cat("skipped SPR ratio timeseries: requires both sprtarg>0 and uncertainty=TRUE.\n")
     }else{
-      sprratiostd <- derived_quants[substring(derived_quants$LABEL,1,8)=="SPRratio",]
-      sprratiostd$Yr <- as.numeric(substring(sprratiostd$LABEL,10))
+      sprratiostd <- derived_quants[substring(derived_quants$Label,1,8)=="SPRratio",]
+      sprratiostd$Yr <- as.numeric(substring(sprratiostd$Label,10))
       sprratiostd$period <- "fore"
       sprratiostd$period[sprratiostd$Yr<=(endyr)] <- "time"
       sprratiostd$upper <- sprratiostd$Value + 1.96*sprratiostd$StdDev
@@ -139,7 +139,7 @@ SSplotSPR <-
       if(3 %in% subplots){
         if(plot) sprfunc3()
         if(print){
-          file <- file.path(plotdir,"/SPR3_ratiointerval.png")
+          file <- "SPR3_ratiointerval.png"
           caption <- "Timeseries of SPR ratio"
           plotinfo <- pngfun(file=file, caption=caption)
           sprfunc3()
@@ -173,8 +173,8 @@ SSplotSPR <-
         }
         # calculate depletion
         depletionseries <- ts$SpawnBio/ts$SpawnBio[1]
-        reldep <- depletionseries[ts$Yr %in% sprseries$Year]/btarg
-        relspr <- (1-sprseries$spr[sprseries$Year <= endyr])/(1-sprtarg)
+        reldep <- depletionseries[ts$Yr %in% sprseries$Yr]/btarg
+        relspr <- (1-sprseries$spr[sprseries$Yr <= endyr])/(1-sprtarg)
         # set axis limits
         xmax <- 1.1*max(reldep)
         ymax <- 1.1*max(1,relspr[!is.na(relspr)])
@@ -194,7 +194,7 @@ SSplotSPR <-
 
         if(plot) phasefunc()
         if(print){
-          file <- file.path(plotdir,"/SPR4_phase.png")
+          file <- "SPR4_phase.png"
           caption <- "Phase plot of biomass ratio vs. SPR ratio"
           plotinfo <- pngfun(file=file, caption=caption)
           phasefunc()
