@@ -1,10 +1,13 @@
 #' Summarize the output from multiple Stock Synthesis models.
-#' 
+#'
 #' Summarize various quantities from the model output collected by
 #' \code{\link{SSgetoutput}} and return them in a list of tables and vectors.
-#' 
-#' 
-#' @param biglist A list of lists created by \code{\link{SSgetoutput}}.
+#'
+#'
+#' @param biglist A list of lists, one for each model. The individual lists can
+#' be created by \code{\link{SS_output}} or the list of lists can be
+#' created by \code{\link{SSgetoutput}} (which iteratively calls
+#' \code{\link{SS_output}}).
 #' @param sizeselfactor A string or vector of strings indicating which elements
 #' of the selectivity at length output to summarize. Default=c("Lsel").
 #' @param ageselfactor A string or vector of strings indicating which elements
@@ -25,7 +28,6 @@
 #' @author Ian Taylor
 #' @export
 #' @seealso \code{\link{SSgetoutput}}
-#' @keywords data manip list
 SSsummarize <- function(biglist,
                         sizeselfactor="Lsel",
                         ageselfactor="Asel",
@@ -50,7 +52,7 @@ SSsummarize <- function(biglist,
   for(imodel in 1:n){
     stats <- biglist[[imodel]]
     parnames <- union(parnames,stats$parameters$Label)
-    dernames <- union(dernames,stats$derived_quants$LABEL)
+    dernames <- union(dernames,stats$derived_quants$Label)
     allyears <- union(allyears,stats$timeseries$Yr)
     likenames <- union(likenames,rownames(stats$likelihoods_used))
   }
@@ -64,6 +66,7 @@ SSsummarize <- function(biglist,
   nsexes     <- NULL
   likelihoods <- likelambdas <- as.data.frame(matrix(NA,nrow=length(likenames),ncol=n))
   likelihoods_by_fleet <- NULL
+  likelihoods_by_tag_group <- NULL
   indices    <- NULL
   sizesel    <- NULL
   agesel     <- NULL
@@ -74,18 +77,19 @@ SSsummarize <- function(biglist,
   startyrs   <- NULL
   endyrs     <- NULL
   SPRratioLabels <- NULL
+  FvalueLabels <- NULL
   sprtargs   <- NULL
   btargs     <- NULL
   minbthreshs <- NULL
   FleetNames <- list()
-  
-  warn <- FALSE # flag for whether filter warning has been printed or not
+  mcmc       <- list()
+  warn       <- FALSE # flag for whether filter warning has been printed or not
 
   # loop over models within biglist
   for(imodel in 1:n){
     stats <- biglist[[imodel]]
     listname <- names(biglist)[imodel]
-    cat("imodel=", imodel, "/", n, sep="")
+    message("imodel=", imodel, "/", n)
 
     # gradient
     maxgrad <- c(maxgrad, stats$maximum_gradient_component)
@@ -96,7 +100,7 @@ SSsummarize <- function(biglist,
     # start and end years
     startyrs <- c(startyrs, stats$startyr)
     endyrs   <- c(endyrs,   stats$endyr)
-    
+
     # size selectivity
     sizeseltemp <- stats$sizeselex
     if(is.null(sizeselfactor)) sizeselfactor <- unique(sizeseltemp$Factor)
@@ -110,7 +114,7 @@ SSsummarize <- function(biglist,
                                 all(names(seltemp_i)==names(sizesel)))){
         sizesel <- rbind(sizesel,seltemp_i)
       }else{
-        cat("problem summarizing size selectivity due to mismatched columns ",
+        warning("problem summarizing size selectivity due to mismatched columns ",
             "(perhaps different bins)\n")
       }
     }
@@ -120,7 +124,7 @@ SSsummarize <- function(biglist,
     ageseltemp  <- stats$ageselex
     if(is.null(ageselfactor)) ageselfactor <- unique(ageseltemp$Factor)
     for(iselfactor in 1:length(ageselfactor)){
-      seltemp_i <- ageseltemp[ageseltemp$factor==ageselfactor[iselfactor],]
+      seltemp_i <- ageseltemp[ageseltemp$Factor==ageselfactor[iselfactor],]
       seltemp_i$imodel <- imodel
       seltemp_i$name <- modelnames[imodel]
       # if agesel is not NULL, then check for whether columns of new addition
@@ -129,32 +133,49 @@ SSsummarize <- function(biglist,
                                all(names(seltemp_i)==names(agesel)))){
         agesel <- rbind(agesel,seltemp_i)
       }else{
-        cat("problem summarizing age selectivity due to mismatched columns ",
-            "(perhaps different bins)\n")
+        warning("problem summarizing age selectivity due to mismatched columns ",
+                "(perhaps different bins)\n")
       }
     }
     rownames(agesel) <- 1:nrow(agesel)
 
-    
+
     ## growth (females only)
     growthtemp <- stats$growthseries
     imorphf <- ifelse(max(stats$morph_indexing$Index)==10,3,1)
     growthtemp <- growthtemp[growthtemp$Morph==imorphf,-(1:4)]
     growth <- cbind(growth, as.numeric(growthtemp[nrow(growthtemp),]))
 
-    ## likelihoods
+    ## likelihoods (total by component)
     liketemp <- stats$likelihoods_used
     for(irow in 1:nrow(liketemp)){
       likelihoods[likenames==rownames(liketemp)[irow], imodel] <- liketemp$values[irow]
       likelambdas[likenames==rownames(liketemp)[irow], imodel] <- liketemp$lambdas[irow]
     }
+    ## likelihoods by fleet
+    # add initial column with model number to table from each model
     liketemp2 <- data.frame(model=imodel,stats$likelihoods_by_fleet)
+    # test for presence of existing table to append to with matching number of columns
     if(is.null(likelihoods_by_fleet) ||
        (ncol(likelihoods_by_fleet)==ncol(liketemp2) &&
          all(names(likelihoods_by_fleet)==names(liketemp2)))){
       likelihoods_by_fleet <- rbind(likelihoods_by_fleet,liketemp2)
     }else{
-      cat("\nproblem summarizing likelihoods by fleet due to mismatched columns\n")
+      warning("problem summarizing likelihoods by fleet due to mismatched columns")
+    }
+
+    ## likelihoods by tag group
+    # add initial column with model number to table from each model
+    if(!is.null(stats$likelihoods_by_tag_group)){
+      liketemp3 <- data.frame(model=imodel,stats$likelihoods_by_tag_group)
+      # test for presence of existing table to append to with matching number of columns
+      if(is.null(likelihoods_by_tag_group) ||
+         (ncol(likelihoods_by_tag_group)==ncol(liketemp3) &&
+            all(names(likelihoods_by_tag_group)==names(liketemp3)))){
+        likelihoods_by_tag_group <- rbind(likelihoods_by_tag_group,liketemp3)
+      }else{
+        warning("problem summarizing likelihoods by fleet due to mismatched columns")
+      }
     }
 
     ## compile parameters
@@ -164,31 +185,42 @@ SSsummarize <- function(biglist,
       parsSD[parnames==parstemp$Label[ipar], imodel] <- parstemp$Parm_StDev[ipar]
       parphases[parnames==parstemp$Label[ipar], imodel] <- parstemp$Phase[ipar]
     }
-    cat(",  N active pars=",sum(!is.na(parstemp$Active_Cnt)),"\n",sep="")
-    
+    message("  N active pars=",sum(!is.na(parstemp$Active_Cnt)))
+
     ## compile derived quantities
     quantstemp <- stats$derived_quants
     for(iquant in 1:nrow(quantstemp)){
-      quants[dernames==quantstemp$LABEL[iquant], imodel] <- quantstemp$Value[iquant]
-      quantsSD[dernames==quantstemp$LABEL[iquant], imodel] <- quantstemp$StdDev[iquant]
+      quants[dernames==quantstemp$Label[iquant], imodel] <- quantstemp$Value[iquant]
+      quantsSD[dernames==quantstemp$Label[iquant], imodel] <- quantstemp$StdDev[iquant]
     }
     SPRratioLabels <- c(SPRratioLabels, stats$SPRratioLabel)
+    FvalueLabels   <- c(FvalueLabels,   stats$F_report_basis)
     sprtargs       <- c(sprtargs,       stats$sprtarg)
     btargs         <- c(btargs,         stats$btarg)
     minbthreshs    <- c(minbthreshs,    stats$minbthresh)
     FleetNames[[imodel]] <- stats$FleetNames
-    
+
     ## indices
     indextemp <- stats$cpue
+    # temporarily remove columns added in SS version 3.30.13 (March 2019)
+    indextemp <- indextemp[!names(indextemp) %in% c("Area","Subseas","Month")] 
     if(is.na(indextemp[[1]][1])){
-      cat("no index data\n")
+      message("no index data")
     }else{
       indextemp$name <- modelnames[imodel]
       indextemp$imodel <- imodel
-      if(is.null(indices) || all(names(indextemp)==names(indices))){
+      if(is.null(indices)){
+        # first pass through with nothing in combined data frame
         indices <- rbind(indices, indextemp)
       }else{
-        cat("problem summarizing indices due to mismatched columns\n")
+        # after indices contains output from at least one model
+        # check that there are equal number of columns with matching names 
+        if(ncol(indextemp) == ncol(indices) &&
+           all(names(indextemp) == names(indices))){
+          indices <- rbind(indices, indextemp)
+        }else{
+          warning("problem summarizing indices due to mismatched columns")
+        }
       }
     }
 
@@ -210,8 +242,14 @@ SSsummarize <- function(biglist,
     if(is.na(SpawnOutputUnits[imodel])){
       SpawnOutputUnits[imodel] <- stats$SpawnOutputUnits
     }
+    # get mcmc values if present
+    if(!is.null(stats$mcmc)){
+      mcmc[[imodel]] <- stats$mcmc
+    }
   } # end loop over models
 
+
+  ### format and process info from the models
   names(pars) <- names(parsSD) <- modelnames
   names(quants) <- names(quantsSD) <- modelnames
   names(likelihoods) <- names(likelambdas) <- modelnames
@@ -235,23 +273,28 @@ SSsummarize <- function(biglist,
     quantsSD$Yr[iquant] <- ifelse(is.null(yr), NA, as.numeric(yr))
   }
 
-
+  SSBrows <- grep("SSB_",quants$Label)
+  SSBexclude <- c(grep("SSB_unfished",quants$Label, ignore.case=TRUE),
+                  grep("SSB_Btgt",quants$Label, ignore.case=TRUE),
+                  grep("SSB_SPR",quants$Label, ignore.case=TRUE),
+                  grep("SSB_MSY", quants$Label, ignore.case=TRUE))
+  SSBrows <- setdiff(SSBrows, SSBexclude)
   # identify spawning biomass parameters
-  SpawnBio <- quants[grep("SPB_",quants$Label), ]
-  SpawnBioSD <- quantsSD[grep("SPB_",quants$Label), ]
+  SpawnBio <- quants[SSBrows, ]
+  SpawnBioSD <- quantsSD[SSBrows, ]
   # add year values for Virgin and Initial years
   minyr <- min(SpawnBio$Yr,na.rm=TRUE)
-  SpawnBio$Yr[grep("SPB_Virgin",SpawnBio$Label)] <- minyr - 2
-  SpawnBio$Yr[grep("SPB_Initial",SpawnBio$Label)] <- minyr - 1
+  SpawnBio$Yr[grep("SSB_Virgin",SpawnBio$Label)] <- minyr - 2
+  SpawnBio$Yr[grep("SSB_Initial",SpawnBio$Label)] <- minyr - 1
   SpawnBioSD$Yr <- SpawnBio$Yr
 
   SpawnBio <- SpawnBio[order(SpawnBio$Yr),]
   SpawnBioSD <- SpawnBioSD[order(SpawnBioSD$Yr),]
   if(any(is.na(SpawnBio[3,]))){
-    cat("Models have different start years, so SpawnBio values in VIRG & INIT yrs are shifted to correct year\n")
-    SpawnBio$Label[1:2] <- c("SPB_Virgin*","SPB_Initial*")
-    SpawnBioSD$Label[1:2] <- c("SPB_Virgin*","SPB_Initial*")
-    for(imodel in 1:n){ 
+    warning("Models have different start years, so SpawnBio values in VIRG & INIT yrs are shifted to correct year")
+    SpawnBio$Label[1:2] <- c("SSB_Virgin*","SSB_Initial*")
+    SpawnBioSD$Label[1:2] <- c("SSB_Virgin*","SSB_Initial*")
+    for(imodel in 1:n){
       if(is.na(SpawnBio[3,imodel])){
         minyr <- min(SpawnBio$Yr[-(1:2)][!is.na(SpawnBio[-(1:2),imodel])]) # first year with value
         SpawnBio[SpawnBio$Yr==minyr-2, imodel] <- SpawnBio[1,imodel]
@@ -263,7 +306,7 @@ SSsummarize <- function(biglist,
       }
     }
   }
-  
+
   SpawnBioLower <- SpawnBioUpper <- SpawnBioSD
   SpawnBioLower[,1:n] <- qnorm(p=lowerCI, mean=as.matrix(SpawnBio[,1:n]),
                                sd=as.matrix(SpawnBioSD[,1:n]))
@@ -280,7 +323,7 @@ SSsummarize <- function(biglist,
   BratioUpper[,1:n] <- qnorm(p=upperCI, mean=as.matrix(Bratio[,1:n]),
                              sd=as.matrix(BratioSD[,1:n]))
 
-  # identify biomass ratio parameters
+  # identify SPR ratio derived quantities
   SPRratio <- quants[grep("^SPRratio_",quants$Label),]
   SPRratioSD <- quantsSD[grep("^SPRratio_",quantsSD$Label),]
 
@@ -289,17 +332,27 @@ SSsummarize <- function(biglist,
                              sd=as.matrix(SPRratioSD[,1:n]))
   SPRratioUpper[,1:n] <- qnorm(p=upperCI, mean=as.matrix(SPRratio[,1:n]),
                              sd=as.matrix(SPRratioSD[,1:n]))
-  
-  
+
+  # identify F derived quantities
+  Fvalue <- quants[grep("^F_",quants$Label),]
+  FvalueSD <- quantsSD[grep("^F_",quantsSD$Label),]
+
+  FvalueLower <- FvalueUpper <- FvalueSD
+  FvalueLower[,1:n] <- qnorm(p=lowerCI, mean=as.matrix(Fvalue[,1:n]),
+                             sd=as.matrix(FvalueSD[,1:n]))
+  FvalueUpper[,1:n] <- qnorm(p=upperCI, mean=as.matrix(Fvalue[,1:n]),
+                             sd=as.matrix(FvalueSD[,1:n]))
+
+
   # identify recruitment parameters and their uncertainty
   recruits <- quants[grep("^Recr_",quants$Label), ]
   recruitsSD <- quantsSD[grep("^Recr_",quantsSD$Label), ]
-  if(length(grep("Recr_Unfished",recruits$Label))>0)
-    recruits <- recruits[-grep("Recr_Unfished",recruits$Label),]
-  if(length(grep("Recr_Unfished",recruitsSD$Label))>0)
-    recruitsSD <- recruitsSD[-grep("Recr_Unfished",recruitsSD$Label),]
+  if(length(grep("Recr_Unfished", recruits$Label, ignore.case=TRUE))>0){
+    recruits <- recruits[-grep("Recr_Unfished",recruits$Label, ignore.case=TRUE),]
+    recruitsSD <- recruitsSD[-grep("Recr_Unfished",recruitsSD$Label, ignore.case=TRUE),]
+  }
   minyr <- min(recruits$Yr,na.rm=TRUE)
-  
+
   recruits$Yr[grep("Recr_Virgin",recruits$Label)] <- minyr - 2
   recruits$Yr[grep("Recr_Initial",recruits$Label)] <- minyr - 1
   recruitsSD$Yr[grep("Recr_Virgin",recruitsSD$Label)] <- minyr - 2
@@ -307,9 +360,9 @@ SSsummarize <- function(biglist,
   recruits <- recruits[order(recruits$Yr),]
   recruitsSD <- recruitsSD[order(recruitsSD$Yr),]
   if(any(is.na(recruits[3,]))){
-    cat("Models have different start years, so recruits values in VIRG & INIT yrs are shifted to correct year\n")
+    warning("Models have different start years, so recruits values in VIRG & INIT yrs are shifted to correct year")
     recruits$Label[1:2] <- c("Recr_Virgin*","Recr_Initial*")
-    for(imodel in 1:n){ 
+    for(imodel in 1:n){
       if(is.na(recruits[3,imodel])){
         minyr <- min(recruits$Yr[-(1:2)][!is.na(recruits[-(1:2),imodel])]) # first year with value
         recruits[recruits$Yr==minyr-2, imodel] <- recruits[1,imodel]
@@ -336,23 +389,42 @@ SSsummarize <- function(biglist,
   # if there are any initial age parameters, figure out what year they're from
   InitAgeRows <- grep("InitAge",pars$Label)
   if(length(InitAgeRows)>0){
-    temp <- unlist(strsplit(pars$Label[InitAgeRows],"InitAge_")) # separate out values from string
-    InitAgeVals <- as.numeric(temp[seq(2,length(temp),2)]) # get odd entries in above separation
+    # separate out values from string
+    temp <- unlist(strsplit(pars$Label[InitAgeRows],"InitAge_"))
+    # get odd entries in above separation
+    InitAgeVals <- as.numeric(temp[seq(2,length(temp),2)])
+    # make empty matrix to store values
     InitAgeYrs <- matrix(NA,nrow=length(InitAgeRows),ncol=n)
+    # loop over models
     for(imodel in 1:n){
+      # get parameters
       modelpars <- pars[,imodel]
+      # get vector of years associated with recdevs
       devyears <- pars$Yr[!is.na(modelpars) & pars$recdev]
-      if(any(!is.na(devyears))) minyr <- min(devyears,na.rm=TRUE) else minyr <- NA
+      # figure out first year of recdevs that already have years figured out
+      if(any(!is.na(devyears))){
+        minyr <- min(devyears,na.rm=TRUE)
+      }else{
+        minyr <- NA
+      }
+      # determine which parameter values are associated with InitAge and not NA
       good <- !is.na(modelpars[InitAgeRows])
-      if(!is.na(minyr) & minyr>0 & any(good)) InitAgeYrs[good,imodel] <- minyr - InitAgeVals[good]
+      # if minyr was not NA, and is above 0 and there are good InitAge values,
+      # then compute the associated year
+      if(!is.na(minyr) & minyr>0 & any(good)){
+        InitAgeYrs[good,imodel] <- minyr - InitAgeVals[good]
+      }
     }
     # check for differences in assignment of initial ages
-    if(any(apply(InitAgeYrs,1,max,na.rm=TRUE) - apply(InitAgeYrs,1,min,na.rm=TRUE) != 0)){
-      cat("warning: years for InitAge parameters are differ between models, use InitAgeYrs matrix\n")
+    if(any(apply(InitAgeYrs,1,max,na.rm=TRUE) -
+             apply(InitAgeYrs,1,min,na.rm=TRUE) != 0)){
+      warning("years for InitAge parameters differ between models,",
+              "use InitAgeYrs matrix")
     }else{
       pars$Yr[InitAgeRows] <- apply(InitAgeYrs,1,max,na.rm=TRUE)
     }
   }else{
+    # no parameters seem to be associated with initial age structure
     InitAgeYrs <- NA
   }
   if(any(pars$recdev)){
@@ -371,7 +443,57 @@ SSsummarize <- function(biglist,
     recdevs <- recdevsSD <- recdevsLower <- recdevsUpper <- NULL
   }
 
-  # function to sort by year 
+
+  # function to merge duplicate rows caused by different parameter labels
+  # that are associated with the same year, such as the recdev for 2016
+  # being called "ForeRecr_2016", "Late_RecrDev_2016", or "Main_RecrDev_2016",
+  # in 3 different models depending on the ending year of each model and the
+  # choice of recdev vector breaks
+  merge.duplicates <- function(x){
+    if(!is.null(x)){
+      if(length(unique(x$Yr)) < length(x$Yr)){
+        # n should be number of models
+        n <- sum(!names(x) %in% c("Label", "Yr"))
+        x2 <- NULL # alternative data.frame
+        for(Yr in unique(x$Yr)){
+          x.Yr <- x[which(x$Yr==Yr),]
+          if(nrow(x.Yr)==1){
+            # if only 1 row associated with this year add to new data.frame
+            x2 <- rbind(x2, x.Yr)
+          }else{
+            # more than 1 row associated with this year
+            # create empty row with matching names
+            newrow <- data.frame(t(rep(NA,n)),
+                                 Label=paste0("Multiple_labels_", Yr), Yr=Yr)
+            names(newrow) <- names(x)
+            # loop over models to pick the (hopefully) unique value among rows
+            for(icol in 1:n){
+              good <- !is.na(x.Yr[ ,icol])
+              if(sum(good) > 1){
+                # warn if more than 1 value
+                warning("multiple recdevs values associated with year =", Yr)
+              }
+              if(sum(good)==1){
+                # put good value into new row
+                newrow[,icol] <- x.Yr[good, icol]
+              }
+              # if there are no good values, this model likely ends prior to Yr
+            }
+            # add new row to new data.frame
+            x2 <- rbind(x2, newrow)
+          } # end test for duplicates for particular year
+        } # end loop over years
+      }else{ # end test for duplicates in general
+        # if no duplicates, just return data.frame
+        x2 <- x
+      }
+    }else{ # test for is.null(x)
+      return(x)
+    }
+    return(x2)
+  }
+
+  # function to sort by year
   sort.fn <- function(x){
     if(!is.null(x)){
       return( x[order(x$Yr),] )
@@ -379,7 +501,7 @@ SSsummarize <- function(biglist,
       return()
     }
   }
-  
+
   mylist <- list()
   mylist$n              <- n
   mylist$npars          <- npars
@@ -396,6 +518,7 @@ SSsummarize <- function(biglist,
   mylist$likelihoods    <- likelihoods
   mylist$likelambdas    <- likelambdas
   mylist$likelihoods_by_fleet <- likelihoods_by_fleet
+  mylist$likelihoods_by_tag_group <- likelihoods_by_tag_group
   mylist$SpawnBio       <- sort.fn(SpawnBio)
   mylist$SpawnBioSD     <- sort.fn(SpawnBioSD)
   mylist$SpawnBioLower  <- sort.fn(SpawnBioLower)
@@ -409,6 +532,11 @@ SSsummarize <- function(biglist,
   mylist$SPRratioLower  <- sort.fn(SPRratioLower)
   mylist$SPRratioUpper  <- sort.fn(SPRratioUpper)
   mylist$SPRratioLabels <- SPRratioLabels
+  mylist$Fvalue         <- sort.fn(Fvalue)
+  mylist$FvalueSD       <- sort.fn(FvalueSD)
+  mylist$FvalueLower    <- sort.fn(FvalueLower)
+  mylist$FvalueUpper    <- sort.fn(FvalueUpper)
+  mylist$FvalueLabels   <- FvalueLabels
   mylist$sprtargs       <- sprtargs
   mylist$btargs         <- btargs
   mylist$minbthreshs    <- minbthreshs
@@ -416,10 +544,10 @@ SSsummarize <- function(biglist,
   mylist$recruitsSD     <- sort.fn(recruitsSD)
   mylist$recruitsLower  <- sort.fn(recruitsLower)
   mylist$recruitsUpper  <- sort.fn(recruitsUpper)
-  mylist$recdevs        <- sort.fn(recdevs)
-  mylist$recdevsSD      <- sort.fn(recdevsSD)
-  mylist$recdevsLower   <- sort.fn(recdevsLower)
-  mylist$recdevsUpper   <- sort.fn(recdevsUpper)
+  mylist$recdevs        <- merge.duplicates(sort.fn(recdevs))
+  mylist$recdevsSD      <- merge.duplicates(sort.fn(recdevsSD))
+  mylist$recdevsLower   <- merge.duplicates(sort.fn(recdevsLower))
+  mylist$recdevsUpper   <- merge.duplicates(sort.fn(recdevsUpper))
   mylist$growth         <- growth
   mylist$sizesel        <- sizesel
   mylist$agesel         <- agesel
@@ -429,7 +557,8 @@ SSsummarize <- function(biglist,
   mylist$upperCI        <- upperCI
   mylist$SpawnOutputUnits <- SpawnOutputUnits
   mylist$FleetNames     <- FleetNames
+  mylist$mcmc           <- mcmc
   #mylist$lbinspop   <- as.numeric(names(stats$sizeselex)[-(1:5)])
-  
+
   return(invisible(mylist))
 } # end function
