@@ -26,7 +26,6 @@
 #' @references Butterworth D.S., Ianelli J.N., Hilborn R. (2003) A statistical
 #' model for stock assessment of southern bluefin tuna with temporal changes in
 #' selectivity. South African Journal of Marine Science 25:331-362.
-#' @keywords hplot
 mountains <- function(zmat, xvec=NULL, yvec=NULL, zscale=3, rev=TRUE,
                       nshades=100,axes=TRUE, xaxs='i', yaxs='i',
                       xlab="", ylab="", las=1, addbox=FALSE, ...){
@@ -61,12 +60,10 @@ mountains <- function(zmat, xvec=NULL, yvec=NULL, zscale=3, rev=TRUE,
 
   ny <- length(yvec)
   if(ny!=nrowz){
-    print("length(yvec) must equal nrow(zmat)",quote=FALSE)
-    return()
+    stop("length(yvec)=",length(yvec), " and nrow(zmat)=", nrow(zmat), " should be equal")
   }
   if(length(xvec)!=ncolz){
-    print("length(xvec) must equal ncol(zmat)",quote=FALSE)
-    return()
+    stop("length(xvec) must equal ncol(zmat)")
   }
 
   zseq <- seq(0, zmax, length=nshades)
